package gvprojects.igel.config;

import gvprojects.igel.model.IgelGameParameters;
import gvprojects.igel.view.Artist;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

/**
 * Dialog allowing users to configure Igel Argern game.
 *
 * @author Zachary Kurmas
 */
// Created  1/6/12 at 9:08 AM
// (C) Zachary Kurmas 2012

public class IgelConfigKurmas {

   private SpinnerNumberModel numPlayers;
   private SpinnerNumberModel hogsPerPlayer;
   private SpinnerNumberModel hogsToWin;
   private SpinnerNumberModel rows;
   private SpinnerNumberModel cols;
   private JComboBox obstacleTypeBox;
   private String[] obstacleTypes = {"Concrete Block", "Deep Pit"};
   private JSpinner rowsSpinner;
   private JSpinner colsSpinner;
   private JLabel fileName;

   private JRadioButton simplePresenter;
   private JRadioButton complexPresenter;

   public enum PresenterType {SIMPLE, COMPLEX}

   /**
    * Code to call when the user presses "Start"
    */
   public interface GameLauncher {
      public void launch(IgelConfigKurmas view);
   }

   private JFrame frame;
   private JButton loadFile;
   private JButton cancel;
   private JButton start;

   private GameLauncher launcher;
   private File file;


   /**
    * Constructor.
    *
    * @param parameters default parameters
    * @param launcher
    */
   public IgelConfigKurmas(IgelGameParameters parameters, GameLauncher launcher) {

      this.launcher = launcher;

      frame = new JFrame();
      frame.add(setupParamPanel(parameters));
      frame.pack();
      //frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.setVisible(true);


      loadFile.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            loadFile();
         }
      });

      cancel.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            cancel();
         }
      });

      start.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            start();
         }
      });
   }

   private JRadioButton defaultPresenter() {
      return complexPresenter;
   }

   private JPanel setupParamPanel(IgelGameParameters parameters) {
      final String[] labels = {"Number of Players", "Hedgehogs Per Player", "Hedgehogs to Win", "Number of Rows",
            "Number of Columns", "Obstacle Type", "Simple Presenter", "Complex Presenter"};

      JPanel p = new JPanel();
      p.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 10));
      p.setLayout(new GridBagLayout());

      GridBagConstraints col0 = new GridBagConstraints();
      col0.gridx = 0;
      col0.anchor = GridBagConstraints.EAST;
      col0.ipadx = 5;
      col0.ipady = 8;

      GridBagConstraints col1 = new GridBagConstraints();
      col1.gridx = 1;
      col1.fill = GridBagConstraints.HORIZONTAL;


      GridBagConstraints sep = new GridBagConstraints();
      sep.gridwidth = 2;
      sep.gridx = 0;
      sep.fill = GridBagConstraints.HORIZONTAL;

      JSpinner numberSpinner;

      // Presenter type
      col0.gridy = 0;
      p.add(new JLabel(labels[6]), col0);

      col1.gridy = 0;
      simplePresenter = new JRadioButton();
      p.add(simplePresenter, col1);

      col0.gridy++;
      p.add(new JLabel(labels[7]), col0);

      col1.gridy++;
      complexPresenter = new JRadioButton();
      p.add(complexPresenter, col1);

      ButtonGroup group = new ButtonGroup();
      group.add(simplePresenter);
      group.add(complexPresenter);

      col1.gridy++;
      col0.gridy++;
      sep.gridy = col0.gridy;
      p.add(new JSeparator(SwingConstants.HORIZONTAL), sep);


      // Num Players
      col0.gridy++;
      p.add(new JLabel(labels[0]), col0);

      numPlayers = new SpinnerNumberModel(parameters.numPlayers, 2, Artist.HEDGEHOG_COLORS.length, 1);
      numberSpinner = new JSpinner(numPlayers);
      col1.gridy++;
      p.add(numberSpinner, col1);

      // Num Hedgehogs per player
      col0.gridy++;
      p.add(new JLabel(labels[1]), col0);

      hogsPerPlayer = new SpinnerNumberModel(parameters.hedgehogsPerPlayer, 2, 20, 1);
      numberSpinner = new JSpinner(hogsPerPlayer);
      col1.gridy++;
      p.add(numberSpinner, col1);

      numberSpinner.addChangeListener(new ChangeListener() {
         public void stateChanged(ChangeEvent changeEvent) {
            hogsToWin.setMaximum(hogsPerPlayer.getNumber().intValue());
            if (hogsToWin.getNumber().intValue() > hogsPerPlayer.getNumber().intValue()) {
               hogsToWin.setValue(hogsPerPlayer.getNumber().intValue());
            }
         }
      });


      // hedgehogs to win
      col0.gridy++;
      p.add(new JLabel(labels[2]), col0);

      hogsToWin = new SpinnerNumberModel(parameters.hedgehogsToWin, 1, parameters.hedgehogsPerPlayer, 1);
      numberSpinner = new JSpinner(hogsToWin);
      col1.gridy++;
      p.add(numberSpinner, col1);

      //
      // Separator
      //
      col1.gridy++;
      col0.gridy++;
      sep.gridy = col0.gridy;
      p.add(new JSeparator(SwingConstants.HORIZONTAL), sep);


      // Num rows
      col0.gridy++;
      p.add(new JLabel(labels[3]), col0);

      rows = new SpinnerNumberModel(parameters.numRows, 2, 10, 1);
      rowsSpinner = new JSpinner(rows);
      col1.gridy++;
      p.add(rowsSpinner, col1);

      // Num Columns
      col0.gridy++;
      p.add(new JLabel(labels[4]), col0);

      cols = new SpinnerNumberModel(parameters.numColumns, 3, 30, 1);
      colsSpinner = new JSpinner(cols);
      col1.gridy++;
      p.add(colsSpinner, col1);

      // Obstacle Type
      col0.gridy++;
      p.add(new JLabel(labels[5]), col0);

      obstacleTypeBox = new JComboBox(obstacleTypes);
      col1.gridy++;
      p.add(obstacleTypeBox, col1);


      // File loaded
      fileName = new JLabel("<none chosen>");
      col0.gridy++;
      p.add(fileName, col0);

      loadFile = new JButton("Choose file");
      col1.gridy++;
      p.add(loadFile, col1);


      col1.gridy++;
      col0.gridy++;
      sep.gridy = col0.gridy;
      p.add(new JSeparator(SwingConstants.HORIZONTAL), sep);

      // Start / Cancel
      start = new JButton("Start game");
      col0.gridy++;
      GridBagConstraints startConstraints = (GridBagConstraints) col1.clone();
      startConstraints.gridx = 0;
      startConstraints.gridy = col0.gridy;
      p.add(start, startConstraints);

      cancel = new JButton("Cancel");
      col1.gridy++;
      p.add(cancel, col1);

      //
      //  Default presenter
      //
      defaultPresenter().setSelected(true);
      return p;
   }


   private void start() {
      launcher.launch(this);
      frame.dispose();
   }

   private void cancel() {
      frame.dispose();
   }

   private void loadFile() {
      JFileChooser chooser = new JFileChooser();
      int returnVal = chooser.showOpenDialog(frame);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
         file = chooser.getSelectedFile();
         rowsSpinner.setEnabled(false);
         //colsSpinner.setVisible(false);
         obstacleTypeBox.setEnabled(false);
         colsSpinner.setEnabled(false);
         fileName.setText(file.getName());
         frame.pack();
      }
   }

   /**
    * Get the current value of all the parameters
    *
    * @return the current value of all the parameters
    */
   public IgelGameParameters getParameters() {
      return new IgelGameParameters(rows.getNumber().intValue(),
            cols.getNumber().intValue(),
            numPlayers.getNumber().intValue(),
            hogsPerPlayer.getNumber().intValue(),
            hogsToWin.getNumber().intValue(),
            obstacleTypes[obstacleTypeBox.getSelectedIndex()]);
   }

   /**
    * Get the currently selected presenter type.
    *
    * @return the currently selected presenter type
    */
   public PresenterType presenterType() {
      if (simplePresenter.isSelected()) {
         return PresenterType.SIMPLE;
      } else {
         return PresenterType.COMPLEX;
      }
   }

   /**
    * Return the selected file
    *
    * @return the selected file
    */
   public File getSelectedFile() {
      return file;
   }
}
