package gvprojects.igel.model;

/**
 * An obstacle that represents a deep pit:  Hedgehogs that fall aren't able to get out until they are in last place.
 *
 * @author Zachary Kurmas
 */
// Created  12/14/11 at 4:54 PM
// (C) Zachary Kurmas 2011

class DeepPitCell extends Cell {

   /**
    * Constructor
    *
    * @param row    the row to which this Cell is assigned
    * @param column the column to which this Cell is assigned
    * @param game   the game
    * @throws IllegalArgumentException if {@code game == null)}
    */
   public DeepPitCell(int row, int column, IgelGameKurmas game) {
      super(row, column, game);
   }

   @Override
   public boolean canEnter(int player) {
      return true;
   }

   // Hedgehogs can leave a deep pit only when all other hedgehogs are at or beyond the current column.
   // In other words, all the cells in previous columns must be empty.
   @Override
   public boolean canLeave() {

      IGameState board = getGame().getState();

      for (int col = 0; col < this.getColumn(); col++) {
         for (int row = 0; row < board.numRows(); row++) {
            ICellState cell = board.getCell(row, col);
            if (cell.size() > 0) {
               return false;
            }
         }
      }
      return true;
   }

   @Override
   public String type() {
      return "DeepPit";
   }
}
