package gvprojects.igel.model;

/**
 * Interface describing the state of a single cell on an Igel Argern game board.
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 1:53 PM
// (C) Zachary Kurmas 2011

public interface ICellState {

   /**
    * return the number of hedgehogs in this cell.
    * @return the number of hedgehogs in this cell.
    */
   int size();

   /**
    * return the number of the player at the given depth
    * @param depth the depth (indexed beginning at 0)
    * @return the number of the player at the given depth
    * @throws IllegalArgumentException if {@code depth < 0} or {@code depth >= size}
    */
   int playerAtDepth(int depth);

   // Idea:  Have this interface add one more method:
   // The presenter then is responsible for generating  a Map between Strings and Artists.
   // The nice thing about this is that (1) it doesn't change the model interface.  Students can add their own cell types
   // without breaking the testing interface, but also have a hook to draw the new types of cells in the view.
   // The view then needs a "setArtistMap" method.

   // If a particular new cell type needs moer info in its state, then the Artist can simply cast the ICellState object
   // into the interface for the particular cell.

   /**
    * The type of this cell.  (Used to choose the correct artist.)
    * @return the type of this cell.
    */
   String type();
}
