package gvprojects.igel.model;

/**
 * Interface describing the state of an IgelArgernFullPresenter game
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 11:02 AM
// (C) Zachary Kurmas 2011

public interface IGameState {

   /**
    * Return the number of players.
    *
    * @return the number of players
    */
   public int numPlayers();

   /**
    * Return the number of hedgehogs per player.
    *
    * @return the number of hedgehogs per player.
    */
   public int hedgehogsPerPlayer();

   /**
    * Return the current game phase.
    *
    * @return the current game phase.
    */
   public Phase phase();

   /**
    * Return the current player's number.
    *
    * @return the current player's number
    */
   int currentPlayer();

   /**
    * return the {@link ICellState} object at the given location
    *
    * @param row    the row (indexed beginning at 0)
    * @param column the column (indexed beginning at 0)
    * @return the {@link ICellState} object at the given location
    * @throws IllegalArgumentException if {@code row} or {@code column} is invalid
    */
   ICellState getCell(int row, int column);

   /**
    * return the number of rows on the board.
    *
    * @return the number of rows on the board.
    */
   int numRows();

   /**
    * return the number of columns on the board.
    *
    * @return the number of columns on the board.
    */
   int numColumns();

   /**
    * Get the current value on the die.
    *
    * @return the current value on the die.
    */
   int currentDieValue();

   /**
    * Get the current message.
    *
    * @return the current message
    */
   String currentMessage();


}
