package gvprojects.igel.model;


/**
 * Describes the basic configuration of a basic game.
 */
// Created  1/12/12 at 5:01 PM
// (C) Zachary Kurmas 2012

public class IgelGameParameters {

   public static final int DEFAULT_NUM_ROWS = 6;
   public static final int DEFAULT_NUM_COLUMNS = 9;
   public static final int DEFAULT_NUM_PLAYERS = 4;
   public static final int DEFAULT_NUM_HOGS = 4;
   public static final int DEFAULT_HOGS_TO_WIN = DEFAULT_NUM_HOGS - 1;
   private static final String DEFAULT_OBSTACLE_TYPE = "DeepPit";

   /**
    * Number of rows on the board.  Ignored when a board configuration file is also specified.
    */
   public int numRows = DEFAULT_NUM_ROWS;

   /**
    * Number of columns on the board. Ignored when a board configuration file is also specified.
    */
   public int numColumns = DEFAULT_NUM_COLUMNS;

   /**
    * Number of players
    */
   public int numPlayers = DEFAULT_NUM_PLAYERS;

   /**
    * Number of hedgehogs controlled by each player
    */
   public int hedgehogsPerPlayer = DEFAULT_NUM_HOGS;

   /**
    * Number of hedgehogs that must cross the finish line for a win.
    */
   public int hedgehogsToWin = DEFAULT_HOGS_TO_WIN;


   /**
    * A {@code String} specifying which type of obstacle should be used for this game. Ignored when a board configuration file is also specified.
    */
   public String obstacleType = DEFAULT_OBSTACLE_TYPE;

   /**
    * Default constructor.
    */
   public IgelGameParameters() {
   }


   /**
    * Constructor
    *
    * @param numRows            the number of rows on the board
    * @param numColumns         the number of columns on the board
    * @param numPlayers         the number of players
    * @param hedgehogsPerPlayer the number of hedgehogs owned by each player*
    * @param hedgehogsToWin     the number of hedgehogs that must cross the finish line before a player wins.
    * @param obstacleType       the type of any obstacle cells (unless specified otherwise)
    */
   public IgelGameParameters(int numRows, int numColumns, int numPlayers, int hedgehogsPerPlayer, int hedgehogsToWin,
                             String obstacleType) {
      this.numRows = numRows;
      this.numColumns = numColumns;
      this.numPlayers = numPlayers;
      this.hedgehogsPerPlayer = hedgehogsPerPlayer;
      this.hedgehogsToWin = hedgehogsToWin;
      this.obstacleType = obstacleType;
   }
}
