package gvprojects.igel.presenter;

import gvprojects.igel.model.*;
import gvprojects.igel.view.*;

import java.awt.*;
import java.util.Map;

/**
 * A presenter that illustrates how to use {@link IgelViewKurmas}
 *
 * @author Zachary Kurmas
 */
// Created  1/15/12 at 5:14 PM
// (C) Zachary Kurmas 2012

public class PresenterToDemoView {

   public PresenterToDemoView() {

      // Create a new game using default parameters
      final IIgelGame model = new IgelGameKurmas(new IgelGameParameters());

      // Place some hedgehogs in the game
      for (int i = 0; i < 6; i++) {
         model.placeHedgehog(i);
      }
      model.placeHedgehog(1);
      model.placeHedgehog(3);
      model.placeHedgehog(5);


      // Add a new Artist for use with IgelViewKurmas
      // This example replaces the default artist for the cells in the Finish column
      // with a new artist that draws a cute picture.
      Map<String, Artist> artistMap = IgelViewKurmas.defaultArtistMap();
      artistMap.put("FinishCell", new SmileyArtist());

      // Create a view configured for the specified game.
      final IIgelView view = new IgelViewKurmas(model.getState(), artistMap);


      view.addCellActionListener(new CellActionListener() {

         private boolean sidewaysButtonEnabled = true;

         public void cellEntered(int row, int column) {
            System.out.printf("The mouse entered cell (%d, %d)\n", row, column);
         }

         public void cellDragEntered(int row, int column, int sourceRow, int sourceColumn) {
            System.out.printf("The mouse was dragged into cell (%d, %d) from (%d, %d)\n", row, column, sourceRow,
                  sourceColumn);
            if (column == 0 && model.getState().phase() == Phase.SETUP && model.canPlaceHedgehog(row)) {
               // Notice the translucent piece in the column containing the mouse.
               view.setProposedMoveTarget(row, column, model.getState().currentPlayer());
            } else {
               view.clearProposedMove();
            }
            view.updateMessage(model.getState().currentMessage());
         }


         public void cellExited(int row, int column) {
            System.out.printf("The mouse exited cell (%d, %d)\n", row, column);
         }

         public void cellClicked(int row, int column) {
            System.out.printf("The mouse clicked in cell (%d, %d)\n", row, column);
            if (column == 0 && model.getState().phase() == Phase.SETUP && model.canPlaceHedgehog(row)) {
               model.placeHedgehog(row);
               view.update(model.getState());
               return;
            }
            view.updateMessage(model.getState().currentMessage());

            switch (row) {
               case 0:
                  // Updates the message at the bottom of the window
                  view.updateMessage(String.format("You clicked in cell (%d, %d)", row, column));
                  break;
               case 1:
                  // Displays a modal dialog box
                  view.displayDialogMessage(String.format("You clicked in cell (%d, %d)", row, column),
                        "FYI:");
                  break;
               case 2:
                  // makes the die appear to roll
                  view.updateDie((column % 6) + 1);
                  break;
               case 3:
                  // toggle the sideways button
                  sidewaysButtonEnabled = !sidewaysButtonEnabled;
                  view.enableSidewaysViewButton(sidewaysButtonEnabled);
                  break;
            }
         }

         public void cellPressed(int row, int column) {
            System.out.printf("The user pressed down in cell (%d, %d)\n", row, column);
         }


         public void cellReleased(int releasedRow, int releasedColumn, int pressedRow, int pressedColumn) {
            System.out.printf("The user released in cell (%d, %d) after having been clicked in (%d, %d)\n",
                  releasedRow, releasedColumn, pressedRow,
                  pressedColumn);
         }
      });
   }

   public static void main(String[] args) {
      new PresenterToDemoView();
   }

   public static class SmileyArtist implements Artist {
      public void draw(CellPanel panel, Graphics g, ICellState state) {
         int inset = (int) (panel.getWidth() * .30);
         g.setColor(Color.blue);
         g.drawOval(10, 10, panel.getWidth() - 20, panel.getHeight() - 20);
         g.fillOval(inset, inset, 5, 5);
         g.fillOval(panel.getWidth() - inset - 5, inset, 5, 5);
      }
   }

}
