package gvprojects.igel.view;

/**
 * Listens for actions on cells in the GUI
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 1:16 PM
// (C) Zachary Kurmas 2011

public interface CellActionListener {
   /**
    * Called when the mouse enters a cell
    *
    * @param row    the row of the cell entered
    * @param column the column of the cell entered
    */
   void cellEntered(int row, int column);


   /**
    * Called when the mouse is dragged into a cell (i.e., enters the cell with the button pushed).
    *
    * @param row          the row of the cell being entered
    * @param column       the column of the cell being entered
    * @param sourceRow    the row of the cell over which the mouse was pressed
    * @param sourceColumn the column of the cell over which the mouse was pressed
    */
   void cellDragEntered(int row, int column, int sourceRow, int sourceColumn);

   /**
    * Called when the mouse exits a cell
    *
    * @param row    the row of the cell exited
    * @param column the column of the cell exited
    */
   void cellExited(int row, int column);


   /**
    * Called when the user clicks on a cell
    *
    * @param row    the row of the cell clicked on
    * @param column the column of the cell clicked on
    */
   void cellClicked(int row, int column);


   /**
    * Called when a user presses the mouse in a cell
    *
    * @param row    the row of the cell clicked on
    * @param column the column of the cell clicked on
    */
   void cellPressed(int row, int column);

   /**
    * Called when the user releases the mouse on a cell.
    * Note: row and column refer to the cell over which the mouse was released.
    * This is different from the default Java Swing behavior that will give the cell over which the mouse was
    * originally pressed.
    *
    * @param releasedRow    the row of the cell over which hte mouse was positioned when released
    * @param releasedColumn the column of the cell over which the mouse was positioned when released
    * @param pressedRow     the row of the cell over which hte mouse was positioned when released
    * @param pressedColumn  the column of the cell over which the mouse was positioned when released
    */
   void cellReleased(int releasedRow, int releasedColumn, int pressedRow, int pressedColumn);

}
