package gvprojects.igel.view;

import gvprojects.igel.model.ICellState;

import java.awt.*;
import java.util.ArrayList;

/**
 * Draws a picture of a corresponding IgelArgernFullPresenter game cell.
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 2:21 PM
// (C) Zachary Kurmas 2011

public class CellPanelArtist implements Artist {

   private static final double TOKEN_SIZE_FACTOR = .75;
   private static final double BOTTOM = .9;
   private static final double DY_FACTOR = .1;
   private static final float TRANSPARANCY_FACTOR = .6f;
   private static final int BORDER_WIDTH = 2;

   //private static final int NONE = -1;


   private static int getDiameter(CellPanel panel) {
      // get the diameter of the circle: 1/2 of the smaller of width or
      // height.
      return (int) ((panel.getHeight() > panel.getWidth()) ? (panel.getWidth() * TOKEN_SIZE_FACTOR)
            : (panel.getHeight() * TOKEN_SIZE_FACTOR));
   }

   ArrayList<Color> getColorList(ICellState cell, CellPanel panel) {
      ArrayList<Color> colors = new ArrayList<Color>();
      int offset = 0;
      if (panel.hideTop()) {
         offset = 1;
      }

      for (int i = 0; i < cell.size() - offset; i++) {
         // print circles from the bottom up.
         colors.add(HEDGEHOG_COLORS[cell.playerAtDepth(cell.size() - i - 1)]);
      }

      if (panel.getProposedEntry() != CellPanel.NONE) {
         colors.add(HEDGEHOG_COLORS[panel.getProposedEntry()]);
      }
      return colors;
   }

   // From Artist
   public void draw(CellPanel panel, Graphics g, ICellState state) {
      Graphics2D g2 = (Graphics2D) (g);

      ArrayList<Color> colors = getColorList(state, panel);

      int diameter = getDiameter(panel);

      int x = (panel.getWidth() - diameter) / 2;
      int y = (int) (panel.getHeight() * (BOTTOM)) - diameter;
      int dy = (int) (panel.getHeight() * DY_FACTOR);

      for (int i = 0; i < colors.size(); i++) {

         // If we have an extra element in the colors array, then we know
         // were are considering adding a new element to this position.
         // That element will always be the last element in the array.
         if (panel.getProposedEntry() != CellPanel.NONE && i == colors.size() - 1) {

            float alpha = TRANSPARANCY_FACTOR;
            int type = AlphaComposite.SRC_OVER;
            AlphaComposite composite = AlphaComposite.getInstance(type,
                  alpha);

            g2.setComposite(composite);
         }

         g.setColor(Color.black);
         g.fillOval(x, y - i * dy, diameter, diameter);

         g.setColor(colors.get(i));
         g.fillOval(x + BORDER_WIDTH, y - i * dy + BORDER_WIDTH, diameter
               - BORDER_WIDTH * 2, diameter - BORDER_WIDTH * 2);
      }
   }
}
