package gvprojects.igel.view;

import gvprojects.igel.model.IGameState;

import javax.swing.*;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;

/**
 * The GUI for a game of Igel Argern
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 10:53 AM
// (C) Zachary Kurmas 2011

public interface IIgelView {

   /**
    * Add the specified listener to each cell in the board.
    *
    * @param cellActionListener the listener
    */
   void addCellActionListener(CellActionListener cellActionListener);


   /**
    * Add the specified mouse listener to the sell at location {@code [row, col]}
    * @param row the row
    * @param col the column
    * @param listener a {@code MouseListener}
    */
   void addMouseListener(int row, int col, MouseListener listener);


   /**
    * Update the view to reflect the current game state.
    *
    * @param state the current game state
    */
   void update(IGameState state);

   /**
    * Specify that the user has proposed moving a hedgehog into the given cell.  (For example,
    * an implementation may draw a translucent piece in the specified square.)
    *
    * @param row    the row
    * @param column the column
    * @param player the player number
    * @throws IllegalArgumentException if either {@code row} or {@code column} are out of range.
    */
   void setProposedMoveTarget(int row, int column, int player);

   /**
    * Cancels any currently displayed proposed moves.
    */
   void clearProposedMove();

   /**
    * Updates the message that is displayed.
    *
    * @param newMessage the new message to display
    */
   void updateMessage(String newMessage);

   /**
    * Update the die to show the results of a recent roll
    *
    * @param newValue the new value for the die
    */
   void updateDie(int newValue);

   /**
    * Mark this cell as the starting point of a proposed move. (An implementation may, for example,
    * choose not to display a piece that the user has selected to move elsewhere.)
    *
    * @param row    the row
    * @param column the column
    * @throws IllegalArgumentException if either {@code row} or {@code column} is not valid
    */
   void setProposedMoveSource(int row, int column);

   /**
    * Add a listener to respond when the user presses the "passSidewaysMove" button.
    *
    * @param actionListener a listener to respond when the user presses the "passSidewaysMove" button.
    */
   void addPassSidewaysMovePressedListener(ActionListener actionListener);

   /**
    * Enable or disable the "Pass Sideways Move" button
    *
    * @param enable {@code true} if the button is to be enabled, {@code false} if the button is to be disabled.
    */
   void enableSidewaysViewButton(boolean enable);

   /**
    * Display a message in a dialog box
    *
    * @param message  the main message (the one that appears in the center of the box)
    * @param title the title of the dialog box
    */
   void displayDialogMessage(String message, String title);

   /**
    * Close the view (i.e., close and dispose of the underlying {@code JFrame}).
    */
   void close();


   /**
    * Add a menu
    * @param menu the menu
    */
   void addMenu(JMenuBar menu);
}
