package gvprojects.igel.view;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

/**
 * A JLabel that draws its text big enough to fill the entire panel
 *
 * @author Zachary Kurmas
 */
// Created  12/6/11 at 3:50 PM
// (C) Zachary Kurmas 2011

class NumberPanel extends JLabel {

   public double factor;
   private static final double DEFAULT_FACTOR = .8;

   /**
    * Constructor
    *
    * @param s the {@code String} to display
    */
   public NumberPanel(String s) {
      this(s, DEFAULT_FACTOR);
   }

   /**
    * Constructor
    *
    * @param s the {@code String} to display
    * @param f the percentage of the panel the string should take up
    */
   public NumberPanel(String s, double f) {
      super(s);
      this.factor = f;

      addComponentListener(new ComponentAdapter() {
         @Override
         public void componentResized(ComponentEvent componentEvent) {
            recalculateFontSize();
         }
      });
   }


   // Note:  re-setting the font calls repaint.  Thus, calculating and setting the font size in paintComponent
   // creates an infinite loop of sorts. To get around that, we use a ComponentListener to listen specifically for
   // resize messages.
   private void recalculateFontSize() {
      Font labelFont = getFont();
      String labelText = getText();

      int stringWidth = getFontMetrics(labelFont).stringWidth(labelText);
      int componentWidth = (int) (getWidth() * factor);

      // Find out how much the font can grow in width.
      double widthRatio = (double) componentWidth / (double) stringWidth;

      int newFontSize = (int) (labelFont.getSize() * widthRatio);
      int componentHeight = (int) (getHeight() * factor);

      // Pick a new font size so it will not be larger than the height of label.
      int fontSizeToUse = Math.min(newFontSize, componentHeight);

      // Set the label's font size to the newly determined size.
      setFont(new Font(labelFont.getName(), Font.PLAIN, fontSizeToUse));

      repaint();
   }
}
