package gvprojects.igel.model;

import edu.gvsu.kurmasz.warszawa.dl.DLException;
import edu.gvsu.kurmasz.warszawa.dl.SimpleFactory;
import org.junit.Test;

import static org.junit.Assert.*;

// Created  1/16/12 at 9:12 PM
// (C) Zachary Kurmas 2012

public class IgelGameTest {

   private static final int[] STANDARD_OBSTACLE_COLUMN = {3, 6, 4, 5, 2, 7};

   private static String className = IgelGameKurmas.class.getName();

   private IIgelGame make(IgelGameParameters parameters) throws DLException {

      return SimpleFactory.make(className, IIgelGame.class, true, parameters);

//      try {
//         return SimpleFactory.make(className, IIgelGame.class, parameters);
//      } catch (DLException e) {
//         if (e.getCause() instanceof InvocationTargetException && e.getCause().getCause() instanceof RuntimeException) {
//            throw (RuntimeException) e.getCause().getCause();
//         } else {
//            throw e;
//         }
//      }
      //return new IgelGameKurmas(parameters);
   }

   private IIgelGame make() throws DLException {
      return SimpleFactory.make(className, IIgelGame.class, true, new IgelGameParameters());
      //return new IgelGameKurmas(new IgelGameParameters());
   }

   /**
    * Place hedgehogs in round-robin order.
    *
    * @param game the game
    * @return an array listing the row in which the last token for each player was placed.
    */
   private int[] placeAllHedgehogs(IIgelGame game) {
      int[] lastMove = new int[game.getState().numPlayers()];
      int totalHedgehogs = game.getState().numPlayers() * game.getState().hedgehogsPerPlayer();
      for (int count = 0; count < totalHedgehogs; count++) {
         lastMove[count % game.getState().numPlayers()] = count % game.getState().numRows();
         game.placeHedgehog(count % game.getState().numRows());
      }
      return lastMove;
   }


   private IgelGameParameters nonStandardParams = new IgelGameParameters(3, 5, 3, 5, 2, "ConcreteBlock");


   private IIgelGame makeAndFinishOne() throws DLException {
      IgelGameParameters params = new IgelGameParameters(6, 5, 2, 3, 2, "ConcreteBlock");
      IIgelGame game = make(params);

      game.setNextRolls(6);

      game.placeHedgehog(0);
      game.placeHedgehog(1);
      game.placeHedgehog(3);
      game.placeHedgehog(2);
      game.placeHedgehog(5);
      game.placeHedgehog(4);


      for (int i = 0; i < 3; i++) {
         game.passSidewaysMove();
         game.moveHedgehog(5, i, i + 1);
      }
      game.setNextRolls(4);
      game.passSidewaysMove();
      game.moveHedgehog(5, 3, 4);
      return game;
   }

   private IIgelGame makeAndWinGame() throws DLException {
      IIgelGame game = makeAndFinishOne();
      for (int i = 0; i < 4; i++) {
         game.passSidewaysMove();
         game.canMoveHedgehog(3, i);
         game.moveHedgehog(3, i, i + 1);
      }
      // At this point, player 0 should have won the game.
      return game;
   }


   // Traps player 0 in the pit in row 5.  At the end of the method,
   // the die value should be 0, and it should be player 0's turn
   private IIgelGame makeAndTrapHedgehog() throws DLException {
      IgelGameParameters params = new IgelGameParameters(6, 5, 2, 3, 2, "DeepPit");
      IIgelGame game = make(params);

      game.setNextRolls(5);

      game.placeHedgehog(0);
      game.placeHedgehog(1);
      game.placeHedgehog(3);
      game.placeHedgehog(2);
      game.placeHedgehog(4);
      game.placeHedgehog(5);

      game.passSidewaysMove();
      game.moveHedgehog(4, 0, 1);

      game.setNextRolls(0);
      game.passSidewaysMove();
      game.moveHedgehog(4, 1, 2);

      return game;
   }


   //
   // Test constructor #1
   //
   @Test
   public void constructorCreatesCorrectBoardSize() throws Throwable {
      IIgelGame game = make(nonStandardParams);
      assertEquals("Incorrect number of rows", nonStandardParams.numRows, game.getState().numRows());
      assertEquals("Incorrect number of columns", nonStandardParams.numColumns, game.getState().numColumns());
   }

   @Test
   public void constructorSetsCorrectNumPlayers() throws Throwable {
      IIgelGame game = make(nonStandardParams);
      assertEquals("Incorrect number of hedgehogs", nonStandardParams.numPlayers, game.getState().numPlayers());
   }

   @Test
   public void constructorSetsCorrectNumHedgehogsPerPlayer() throws Throwable {
      IIgelGame game = make(nonStandardParams);
      assertEquals("Incorrect hogs per player", nonStandardParams.hedgehogsPerPlayer,
            game.getState().hedgehogsPerPlayer());
   }

   @Test
   public void constructorSetsDefaultObstacleLocationAndType() throws Throwable {
      IIgelGame game = make(nonStandardParams);
      IGameState state = game.getState();
      for (int row = 0; row < nonStandardParams.numRows; row++) {
         for (int col = 0; col < nonStandardParams.numColumns - 1; col++) {
            String place = String.format("[%d, %d]", row, col);
            if (col == STANDARD_OBSTACLE_COLUMN[row]) {
               assertEquals("Wrong type for obstacle at " + place,
                     "ConcreteBlock", state.getCell(row, col).type());
            } else {
               assertEquals("Wrong type for \"normal\" cell at" + place,
                     "Cell", state.getCell(row, col).type());
            }
         }
         assertEquals("Wrong type for \"finish\" cell at end of row " + row,
               "FinishCell", state.getCell(row, nonStandardParams.numColumns - 1).type());
      }
   }

   @Test
   public void constructorSetsPhaseToSetup() throws Throwable {
      IIgelGame game = make(nonStandardParams);
      assertEquals(Phase.SETUP, game.getState().phase());
   }

   @Test
   public void constructorSetsCurrentPlayerto0() throws Throwable {
      IIgelGame game = make(nonStandardParams);
      assertEquals(0, game.getState().currentPlayer());
   }

   // The Warszawa.dl library currently doesn't support passing null as a parameter
//   @Test(expected = IllegalArgumentException.class)
//   public void constructorComplainsIfParamsIsNull() throws Throwable {
//      make(null);
//   }


   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfZeroRows() throws Throwable {
      make(new IgelGameParameters(0, 5, 3, 5, 2, "ConcreteBlock"));
   }

   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfNegativeRows() throws Throwable {
      make(new IgelGameParameters(-1, 5, 3, 5, 2, "ConcreteBlock"));
   }


   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfZeroColumns() throws Throwable {
      make(new IgelGameParameters(6, 0, 3, 5, 2, "ConcreteBlock"));
   }

   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfNegativeColumns() throws Throwable {
      make(new IgelGameParameters(6, -1, 3, 5, 2, "ConcreteBlock"));
   }


   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfZeroPlayers() throws Throwable {
      make(new IgelGameParameters(6, 5, 0, 5, 2, "ConcreteBlock"));
   }

   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfNegativePlayers() throws Throwable {
      make(new IgelGameParameters(6, 5, -1, 5, 2, "ConcreteBlock"));
   }


   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfZeroHedgehogs() throws Throwable {
      make(new IgelGameParameters(6, 5, 3, 0, 2, "ConcreteBlock"));
   }

   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfNegativeHedegehogs() throws Throwable {
      make(new IgelGameParameters(6, 5, 3, -1, 2, "ConcreteBlock"));
   }


   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfZeroHedgehogsPerPlayer() throws Throwable {
      make(new IgelGameParameters(6, 5, 3, 5, 0, "ConcreteBlock"));
   }

   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfNegativeHedegehogsPerPlayer() throws Throwable {
      make(new IgelGameParameters(6, 5, 3, 5, -1, "ConcreteBlock"));
   }

   @Test(expected = IllegalArgumentException.class)
   public void constructorComplainsIfTooManyHedegehogsPerPlayer() throws Throwable {
      make(new IgelGameParameters(6, 5, 3, 5, 6, "ConcreteBlock"));
   }

   @Test
   public void hogsPerPlayerMayEqualNumPlayers() throws Throwable {
      make(new IgelGameParameters(6, 5, 3, 3, 3, "ConcreteBlock"));
   }


   @Test(expected = IllegalArgumentException.class)
   public void unrecognizedTypeNotAllowed() throws Throwable {
      make(new IgelGameParameters(6, 5, 3, 3, 3, "xlkjpiuewr  $$$$ $^*&^"));
   }

   //
   // Can place hedgehog
   //

   @Test
   public void canPlaceHedgehogAnywhereInEmptyRow() throws Throwable {
      IIgelGame game = make();
      for (int count = 0; count < game.getState().numRows(); count++) {
         assertTrue(game.canPlaceHedgehog(count));
      }
   }

   @Test
   public void canPlaceHedgehogsInEmptySpacesInFirstRow() throws Throwable {
      IIgelGame game = make();
      for (int count = 0; count < game.getState().numRows(); count += 2) {
         game.placeHedgehog(count);
      }
      for (int count = 1; count < game.getState().numRows(); count += 2) {
         assertTrue("Problem in row " + count, game.canPlaceHedgehog(count));
      }
   }

   @Test
   public void cannotPlaceHedgehogsInFullSpacesInFirstRow() throws Throwable {
      IIgelGame game = make();
      for (int count = 0; count < game.getState().numRows(); count += 2) {
         game.placeHedgehog(count);
      }
      for (int count = 0; count < game.getState().numRows(); count += 2) {
         assertFalse("Problem in row " + count, game.canPlaceHedgehog(count));
      }
   }


   @Test
   public void canPlaceHedgeHogAnywhereWhenFirstRowFull() throws Throwable {
      IIgelGame game = make();
      for (int count = 0; count < game.getState().numRows(); count++) {
         game.placeHedgehog(count);
      }
      for (int count = 0; count < game.getState().numRows(); count++) {
         assertTrue("Problem in row " + count, game.canPlaceHedgehog(count));
      }
   }


   @Test
   public void canPlaceHedgehogsInEmptySpacesInSecondRow() throws Throwable {
      IIgelGame game = make();
      // fill the first row
      for (int count = 0; count < game.getState().numRows(); count += 1) {
         game.placeHedgehog(count);
      }

      // partially fill the second row
      for (int count = 0; count < game.getState().numRows(); count += 2) {
         game.placeHedgehog(count);
      }

      // Check for placement in "empty" spaces
      for (int count = 1; count < game.getState().numRows(); count += 2) {
         assertTrue("Problem in row " + count, game.canPlaceHedgehog(count));
      }
   }


   @Test
   public void cannotPlaceHedgehogsInFullSpacesInSecondRow() throws Throwable {
      IIgelGame game = make();
      // fill the first row
      for (int count = 0; count < game.getState().numRows(); count += 1) {
         game.placeHedgehog(count);
      }

      // partially fill the second row
      for (int count = 0; count < game.getState().numRows(); count += 2) {
         game.placeHedgehog(count);
      }

      // check for placement in "full" spaces in second row
      for (int count = 0; count < game.getState().numRows(); count += 2) {
         assertFalse("Problem in row " + count, game.canPlaceHedgehog(count));
      }
   }


   @Test
   public void canPlaceHedgeHogAnywhereWhenFirstTwoRowsFull() throws Throwable {
      IIgelGame game = make();
      int numRows = game.getState().numRows();
      for (int count = 0; count < numRows * 2; count++) {
         game.placeHedgehog(count % numRows);
      }
      for (int count = 0; count < numRows; count++) {
         assertTrue("Problem in row " + count, game.canPlaceHedgehog(count));
      }
   }


   @Test
   public void canPlaceHedgehogsInEmptySpacesInThirdRow() throws Throwable {
      IIgelGame game = make();
      int numRows = game.getState().numRows();
      // fill the first two rows
      for (int count = 0; count < numRows * 2; count += 1) {
         game.placeHedgehog(count % numRows);
      }

      // partially fill the second row
      for (int count = 0; count < numRows; count += 2) {
         game.placeHedgehog(count);
      }

      // Check for placement in "empty" spaces
      for (int count = 1; count < numRows; count += 2) {
         assertTrue("Problem in row" + count, game.canPlaceHedgehog(count));
      }
   }


   @Test
   public void cannotPlaceHedgehogsInFullSpacesInThirdRow() throws Throwable {
      IIgelGame game = make();
      int numRows = game.getState().numRows();
      // fill the first two rows
      for (int count = 0; count < numRows * 2; count += 1) {
         game.placeHedgehog(count % numRows);
      }

      // partially fill the second row
      for (int count = 0; count < numRows; count += 2) {
         game.placeHedgehog(count);
      }

      // check for placement in "full" spaces in second row
      for (int count = 0; count < numRows; count += 2) {
         assertFalse("Problem in row" + count, game.canPlaceHedgehog(count));
      }
   }

   @Test(expected = IllegalMoveException.class)
   public void canPlaceHedgehogThrowsExceptionIfInSidewaysPhase() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      assertEquals(Phase.SIDEWAYS, game.getState().phase());
      game.canPlaceHedgehog(0);
   }

   @Test(expected = IllegalMoveException.class)
   public void canPlaceHedgehogThrowsExceptionIfInMovePhase() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      assertEquals(Phase.SIDEWAYS, game.getState().phase());
      game.passSidewaysMove();
      assertEquals(Phase.MOVE, game.getState().phase());
      game.canPlaceHedgehog(0);
   }

   @Test(expected = IllegalMoveException.class)
   public void canPlaceHedgehogThrowsExceptionIfGameCompleted() throws Throwable {
      IIgelGame game = makeAndWinGame();
      game.canPlaceHedgehog(0);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canPlaceHedgehogTrhowsExceptionIfRowTooSmall() throws Throwable {
      IIgelGame game = make();
      game.placeHedgehog(-1);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canPlaceHedgehogThowsExceptionIfRowTooLarge() throws Throwable {
      IIgelGame game = make();
      game.placeHedgehog(game.getState().numRows());
   }


   //
   // place hedgehog
   //
   @Test
   public void hedgehogPlacedInFirstRowAsExpected() throws Throwable {
      IIgelGame game = make();
      game.placeHedgehog(1);
      game.placeHedgehog(3);
      game.placeHedgehog(4);
      IGameState state = game.getState();

      assertEquals(0, state.getCell(1, 0).playerAtDepth(0));
      assertEquals(1, state.getCell(3, 0).playerAtDepth(0));
      assertEquals(2, state.getCell(4, 0).playerAtDepth(0));
   }

   @Test
   public void hedgehogPlacedInSecondRowAsExpected() throws Throwable {
      IIgelGame game = make();

      // fill the first row
      for (int count = 0; count < game.getState().numRows(); count += 1) {
         game.placeHedgehog(count);
      }
      game.placeHedgehog(1);
      game.placeHedgehog(3);
      game.placeHedgehog(4);

      IGameState state = game.getState();
      assertEquals(2, state.getCell(1, 0).playerAtDepth(0));
      assertEquals(3, state.getCell(3, 0).playerAtDepth(0));
      assertEquals(0, state.getCell(4, 0).playerAtDepth(0));
   }

   @Test
   public void hedgehogPlacedInThirdRowAsExpected() throws Throwable {
      IIgelGame game = make();

      // fill the first two rows
      for (int count = 0; count < game.getState().numRows() * 2; count += 1) {
         game.placeHedgehog(count % game.getState().numRows());
      }
      game.placeHedgehog(1);
      game.placeHedgehog(3);
      game.placeHedgehog(4);

      IGameState state = game.getState();
      assertEquals(0, state.getCell(1, 0).playerAtDepth(0));
      assertEquals(1, state.getCell(3, 0).playerAtDepth(0));
      assertEquals(2, state.getCell(4, 0).playerAtDepth(0));
   }

   @Test
   public void placeHedgehogStacksHedgehogs() throws Throwable {
      IIgelGame game = make();

      // fill the first two rows
      for (int count = 0; count < game.getState().numRows() * 2; count += 1) {
         game.placeHedgehog(count % game.getState().numRows());
      }
      game.placeHedgehog(1);

      IGameState state = game.getState();
      assertEquals(0, state.getCell(1, 0).playerAtDepth(0));
      assertEquals(3, state.getCell(1, 0).playerAtDepth(1));
      assertEquals(1, state.getCell(1, 0).playerAtDepth(2));
   }

   @Test(expected = IllegalMoveException.class)
   public void placeHedgehogThrowsExceptionIfHedgehogNotPlaceable() throws Throwable {
      IIgelGame game = make();
      game.placeHedgehog(0);
      game.placeHedgehog(0);
   }

   @Test(expected = IllegalMoveException.class)
   public void placeHedgehogThrowsExceptionIfSidewaysPhase() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      assertEquals(Phase.SIDEWAYS, game.getState().phase());
      game.placeHedgehog(0);
   }

   @Test(expected = IllegalMoveException.class)
   public void placeHedgehogThrowsExceptionIfForwardPhase() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      assertEquals(Phase.SIDEWAYS, game.getState().phase());
      game.passSidewaysMove();
      assertEquals(Phase.MOVE, game.getState().phase());
      game.placeHedgehog(0);
   }

   @Test(expected = IllegalMoveException.class)
   public void placeHedgehogThrowsExceptionIfEndOfGame() throws Throwable {
      IIgelGame game = makeAndWinGame();
      game.placeHedgehog(0);
   }


   @Test
   public void placeHedgehogIncrementsCurrentPlayer() throws Throwable {
      IIgelGame game = make();
      game.placeHedgehog(0);
      assertEquals(1, game.getState().currentPlayer());
      game.placeHedgehog(4);
      assertEquals(2, game.getState().currentPlayer());
      game.placeHedgehog(2);
      assertEquals(3, game.getState().currentPlayer());
   }

   @Test
   public void placeHedgehogChangesPhaseWhenLastHedgehogPlaced() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      assertEquals(Phase.SIDEWAYS, game.getState().phase());
   }

   @Test
   public void gameDoesNotChangePhaseUntilAllHedgehogsPlaced() throws Throwable {
      IIgelGame game = make();
      int totalHedgehogs = game.getState().numPlayers() * game.getState().hedgehogsPerPlayer();
      for (int count = 0; count < totalHedgehogs - 1; count++) {
         game.placeHedgehog(count % game.getState().numRows());
         assertEquals(Phase.SETUP, game.getState().phase());
      }
   }

   //
   // can slide hedgehog (fromRow, fromColumn)
   //
   @Test
   public void canSlideOwnHedgehog() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      for (int i = 0; i < game.getState().numRows(); i++) {
         if (game.getState().getCell(i, 0).playerAtDepth(0) == 0) {
            assertTrue("Problem in row " + i, game.canSlideHedgehog(i, 0));
         }
      }
   }

   @Test
   public void canNotSlideOtherPlayersHedgehog() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      for (int i = 0; i < game.getState().numRows(); i++) {
         if (game.getState().getCell(i, 0).playerAtDepth(0) != i)
            assertFalse(game.canSlideHedgehog(i, 0));
      }
   }

   @Test
   public void cannotSlideHedgehogIfNotAtTop() throws Throwable {
      IIgelGame game = make(new IgelGameParameters(2, 3, 2, 2, 2, "ConcreteBlock"));
      game.placeHedgehog(0);
      game.placeHedgehog(1);
      game.placeHedgehog(1);
      game.placeHedgehog(0);
      // Player 0 has a hedgehog at 0,0, just not on top.
      assertFalse(game.canSlideHedgehog(0, 0));
   }

   @Test
   public void cannotSlideHedgehogIfCellEmpty() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      for (int i = 0; i < game.getState().numRows(); i++) {
         assertFalse("Row " + i, game.canSlideHedgehog(i, 1));
      }
   }

   @Test
   public void cannotSlideTrappedHedgehog() throws Throwable {
      IIgelGame game = makeAndTrapHedgehog();
      assertFalse(game.canSlideHedgehog(4, 2));
   }


   @Test(expected = IllegalMoveException.class)
   public void canSlideHedgehogThrowsExceptionIfStartPhase() throws Throwable {
      IIgelGame game = make();
      game.canSlideHedgehog(0, 0);
   }


   @Test(expected = IllegalMoveException.class)
   public void canSlideHedgehogThrowsExceptionIfMovePhase() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.slideHedgehog(0, 0, 1);
      game.canSlideHedgehog(0, 0);
   }

   @Test(expected = IllegalMoveException.class)
   public void canSlideHedgehogThrowsExceptionIfEndOfGame() throws Throwable {
      IIgelGame game = makeAndWinGame();
      game.canSlideHedgehog(0, 0);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfRowTooSmall() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(-1, 0);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfRowTooLarge() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(game.getState().numRows(), 0);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfColumnTooSmall() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(0, -1);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfColumnTooLarge() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(0, game.getState().numColumns());
   }

   @Test
   public void cannotSlideHedgehogIfAtFinish() throws Throwable {
      IIgelGame game = makeAndFinishOne();
      assertFalse(game.canSlideHedgehog(5, 4));
   }

   //
   // can slide hedgehog (from, to)
   //

   private static void checkBothSlidesTrue(IIgelGame game, int fromRow, int fromColumn) {
      if (fromRow - 1 >= 0) {
         assertTrue("Problem in row " + fromRow, game.canSlideHedgehog(fromRow, fromColumn, fromRow - 1));
      }
      if (fromRow + 1 < game.getState().numRows()) {
         assertTrue("Problem in row " + fromRow, game.canSlideHedgehog(fromRow, fromColumn, fromRow + 1));
      }
   }

   private static void checkBothSlidesFalse(IIgelGame game, int fromRow, int fromColumn) {
      if (fromRow - 1 >= 0) {
         assertFalse("Problem in row " + fromRow, game.canSlideHedgehog(fromRow, fromColumn, fromRow - 1));
      }
      if (fromRow + 1 < game.getState().numRows()) {
         assertFalse("Problem in row " + fromRow, game.canSlideHedgehog(fromRow, fromColumn, fromRow + 1));
      }
   }

   @Test
   public void canSlideOwnHedgehog_fromTo() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      for (int i = 0; i < game.getState().numRows(); i++) {
         if (game.getState().getCell(i, 0).playerAtDepth(0) == 0) {
            checkBothSlidesTrue(game, i, 0);
         }
      }
   }

   @Test
   public void canNotSlideOtherPlayersHedgehog_fromTo() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      for (int i = 0; i < game.getState().numRows(); i++) {
         if (game.getState().getCell(i, 0).playerAtDepth(0) != i)
            checkBothSlidesFalse(game, i, 0);
      }
   }

   @Test
   public void cannotSlideHedgehogIfNotAtTop_toFrom() throws Throwable {
      IIgelGame game = make(new IgelGameParameters(3, 3, 2, 3, 2, "ConcreteBlock"));
      game.placeHedgehog(1);
      game.placeHedgehog(0);
      game.placeHedgehog(2);
      game.placeHedgehog(1);
      game.placeHedgehog(0);
      game.placeHedgehog(2);
      // Player 0 has a hedgehog at 1,0, just not on top.
      assertFalse(game.canSlideHedgehog(1, 0, 0));
      assertFalse(game.canSlideHedgehog(1, 0, 2));
   }

   private void printMessage(IIgelGame game) {
      System.out.println(game.getState().currentMessage());
   }

   @Test
   public void cannotSlideHedgehogIfCellEmpty_fromTo() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      for (int i = 0; i < game.getState().numRows(); i++) {
         checkBothSlidesFalse(game, i, 1);
      }
   }

   @Test
   public void cannotSlideTrappedHedgehog_fromTo() throws Throwable {
      IIgelGame game = makeAndTrapHedgehog();
      assertFalse(game.canSlideHedgehog(4, 2, 3));
      assertFalse(game.canSlideHedgehog(4, 2, 5));
   }


   @Test(expected = IllegalMoveException.class)
   public void canSlideHedgehogThrowsExceptionIfStartPhase_fromTo() throws Throwable {
      IIgelGame game = make();
      game.canSlideHedgehog(0, 0, 1);
   }


   @Test(expected = IllegalMoveException.class)
   public void canSlideHedgehogThrowsExceptionIfMovePhase_fromTo() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.slideHedgehog(0, 0, 1);
      game.canSlideHedgehog(0, 1, 0);
   }

   @Test(expected = IllegalMoveException.class)
   public void canSlideHedgehogThrowsExceptionIfEndOfGame_fromTo() throws Throwable {
      IIgelGame game = makeAndWinGame();
      game.canSlideHedgehog(0, 0, 1);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfRowTooSmall_fromTo() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(-1, 0, 0);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfRowTooLarge_fromTo() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(game.getState().numRows(), 0, game.getState().numRows() - 1);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfTargetRowTooSmall_fromTo() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(0, 0, -1);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfTargetRowTooSmall_fromTo2() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(-1, 0, -1);
   }


   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfTargetRowTooLarge_fromTo() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(game.getState().numRows() - 1, 0, game.getState().numRows());
   }

   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfTargetRowTooLarge_fromTo2() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(game.getState().numRows(), 0, game.getState().numRows());
   }


   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfColumnTooSmall_fromTo() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(0, -1, 1);
   }

   @Test(expected = IllegalArgumentException.class)
   public void canSlideHedgehogThrowsExceptionIfColumnTooLarge_fromTo() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      game.canSlideHedgehog(0, game.getState().numColumns(), 1);
   }

   @Test
   public void cannotSlideHedgehogIfAtFinish_fromTo() throws Throwable {
      IIgelGame game = makeAndFinishOne();
      assertFalse(game.canSlideHedgehog(5, 4, 4));
   }

   @Test
   public void canOnlySlideToNeighboringRow() throws Throwable {
      IIgelGame game = make();
      placeAllHedgehogs(game);
      assertFalse(game.canSlideHedgehog(0, 0, 2));
   }

   @Test
   public void cannotSlideHedgehogIntoConcreteBlock() throws Throwable {
      IgelGameParameters params = new IgelGameParameters(6, 5, 2, 3, 2, "ConcreteBlock");
      IIgelGame game = make(params);

      game.setNextRolls(4);

      game.placeHedgehog(0);
      game.placeHedgehog(1);
      game.placeHedgehog(3);
      game.placeHedgehog(2);
      game.placeHedgehog(4);
      game.placeHedgehog(5);

      game.passSidewaysMove();
      game.moveHedgehog(3, 0, 1);

      game.setNextRolls(0);
      game.passSidewaysMove();
      game.moveHedgehog(3, 1, 2);

      assertFalse(game.canSlideHedgehog(3, 2, 4));
   }

   @Test
   public void canSlideHedgehogIntoDeepPit() throws Throwable {
      IgelGameParameters params = new IgelGameParameters(6, 5, 2, 3, 2, "DeepPit");
      IIgelGame game = make(params);

      game.setNextRolls(4);

      game.placeHedgehog(0);
      game.placeHedgehog(1);
      game.placeHedgehog(3);
      game.placeHedgehog(2);
      game.placeHedgehog(4);
      game.placeHedgehog(5);

      game.passSidewaysMove();
      game.moveHedgehog(3, 0, 1);

      game.setNextRolls(0);
      game.passSidewaysMove();
      game.moveHedgehog(3, 1, 2);

      assertTrue(game.canSlideHedgehog(3, 2, 4));
   }

   private IIgelGame setupDeepPitTests() throws DLException {
      IgelGameParameters params = new IgelGameParameters(6, 5, 2, 3, 2, "DeepPit");
      IIgelGame game = make(params);

      game.setNextRolls(4);

      game.placeHedgehog(0);
      game.placeHedgehog(1);
      game.placeHedgehog(3);
      game.placeHedgehog(2);
      game.placeHedgehog(4);
      game.placeHedgehog(5);

      game.passSidewaysMove();
      game.moveHedgehog(3, 0, 1);

      game.setNextRolls(1);
      game.passSidewaysMove();
      game.moveHedgehog(3, 1, 2);

      // player 0
      game.slideHedgehog(3, 2, 4);
      game.moveHedgehog(0, 0, 1);

      // player 1
      game.setNextRolls(2);
      game.passSidewaysMove();
      game.moveHedgehog(0, 1, 2);

      return game;

   }

   @Test
   public void cannotSlideHedgehogsOutOfDeepPit() throws Throwable {
      IIgelGame game = setupDeepPitTests();
      // player 0
      assertFalse(game.canSlideHedgehog(4, 2));
   }

   @Test
   public void canSlideHedgehogsOutOfDeepPitIfAtEnd() throws Throwable {
      IIgelGame game = setupDeepPitTests();

      // Hedgehog is stuck
      assertFalse(game.canSlideHedgehog(4, 2));

      //
      // Move row 2 (as labeled on board)
      //

      game.passSidewaysMove();
      game.moveHedgehog(1, 0, 1);


      game.setNextRolls(3);
      game.passSidewaysMove();
      game.moveHedgehog(1, 1, 2);

      //
      // Move row 3 (as labeled on the board)
      //

      game.passSidewaysMove();
      game.moveHedgehog(2, 0, 1);


      game.setNextRolls(6);
      game.passSidewaysMove();
      game.moveHedgehog(2, 1, 2);

      //
      // Move row 6 (as labeled on the board)
      //

      game.passSidewaysMove();
      game.moveHedgehog(5, 0, 1);


      game.setNextRolls(5);
      game.passSidewaysMove();
      game.moveHedgehog(5, 1, 2);

      //
      // Move row 5 (as labeled on the board)
      //

      // Hedgehog should still be stuck
      assertFalse(game.canSlideHedgehog(4, 2));

      //player 0
      game.setNextRolls(1);
      game.passSidewaysMove();
      game.moveHedgehog(4, 0, 1);

      // Don't put player 1 in the pit.

      // player 1
      game.setNextRolls(2);
      game.passSidewaysMove();
      game.moveHedgehog(0, 2, 3);

      // player 0
      // Hedgehog should still be stuck
      assertFalse(game.canSlideHedgehog(4, 2));

      game.setNextRolls(4);
      game.slideHedgehog(4, 1, 3);
      game.moveHedgehog(1, 2, 3);

      // player 1
      assertFalse(game.canSlideHedgehog(4, 2));    // wrong player's turn

      game.passSidewaysMove();
      game.moveHedgehog(3, 1, 2);

      // Now our hedgehog should be free
      assertTrue(game.canSlideHedgehog(4, 2));

   }

   public static void setClass(String name) {
      className = name;
   }
}
